@@ -25,6 +25,7 @@ SOFTWARE.
 #include "dynamic_program.h"
 #include "main.h"
 #include <R.h>
+#include <R_ext/Random.h>
 #include <Rcpp.h>
 #include <Rinternals.h>
 #include <vector>
@@ -60,7 +61,7 @@ QMethodInput initializeQMethodInput(const std::string &EMAlg, int samples, int s
     {
         inputParams.monteCarloIter = monte_iter;
         inputParams.errorThreshold = monte_error;
-        inputParams.simulationMethod = monte_method.c_str();
+        inputParams.simulationMethod = strdup(monte_method.c_str());
     }
 
     return inputParams;
@@ -75,6 +76,10 @@ void cleanGlobals(const std::string &EMAlg, bool everything)
         cleanHitAndRun();
     else if (EMAlg == "exact")
         cleanExact();
+    // else if (EMAlg == "mult")
+    //{
+    //    cleanMultinomial();
+    //}
 }
 
 // ---- Set Parameters ---- //
@@ -117,7 +122,11 @@ Rcpp::List EMAlgorithmFull(Rcpp::String em_method, Rcpp::String probability_meth
 
     Matrix Pnew =
         EMAlgoritm(&pIn, EMAlg.c_str(), stopping_threshold[0], log_stopping_threshold[0], maximum_iterations[0],
-                   maximum_seconds[0], verbose[0], &timeIter, &totalIter, &logLLarr, &qvalue, &finish, inputParams);
+                   maximum_seconds[0], verbose[0], &timeIter, &totalIter, &logLLarr, &qvalue, &finish, &inputParams);
+    if (inputParams.simulationMethod != nullptr)
+    {
+        free((void *)inputParams.simulationMethod);
+    }
 
     // ---- Create human-readable stopping reason ---- //
     std::vector<std::string> stop_reasons = {"Converged", "Maximum time reached", "Maximum iterations reached"};
@@ -166,7 +175,11 @@ Rcpp::NumericMatrix bootstrapAlg(Rcpp::NumericMatrix candidate_matrix, Rcpp::Num
 
     Matrix sdResult =
         bootstrapA(&XR, &WR, nboot[0], EMAlg.c_str(), probabilityM.c_str(), stopping_threshold[0],
-                   log_stopping_threshold[0], maximum_iterations[0], maximum_seconds[0], verbose[0], inputParams);
+                   log_stopping_threshold[0], maximum_iterations[0], maximum_seconds[0], verbose[0], &inputParams);
+    if (inputParams.simulationMethod != nullptr)
+    {
+        free((void *)inputParams.simulationMethod);
+    }
 
     // Convert to R's matrix
     Rcpp::NumericMatrix output(sdResult.rows, sdResult.cols);
@@ -209,14 +222,17 @@ Rcpp::List groupAgg(Rcpp::String sd_statistic, Rcpp::NumericVector sd_threshold,
 
     // We'll hold the boundary indices here
     int G = WR.cols;
-    int *cuttingBuffer = new int[G - 1];
+    int *cuttingBuffer = new int[G];
     int usedCuts = 0; // how many boundaries we actually use
     bool bestResult = false;
     Matrix sdResult =
         aggregateGroups(&XR, &WR, cuttingBuffer, &usedCuts, &bestResult, sd_threshold[0], aggMet.c_str(), feasible[0],
                         nboot[0], probabilityM.c_str(), EMAlg.c_str(), stopping_threshold[0], log_stopping_threshold[0],
-                        maximum_iterations[0], maximum_seconds[0], verbose[0], inputParams);
-
+                        maximum_iterations[0], maximum_seconds[0], verbose[0], &inputParams);
+    if (inputParams.simulationMethod != nullptr)
+    {
+        free((void *)inputParams.simulationMethod);
+    }
     // Convert to R's matrix
     Rcpp::NumericMatrix output(sdResult.rows, sdResult.cols);
 
@@ -294,7 +310,12 @@ Rcpp::List groupAggGreedy(Rcpp::String sd_statistic, Rcpp::NumericVector sd_thre
     Matrix greedyP = aggregateGroupsExhaustive(
         &XR, &WR, boundaries, &numCuts, set_method.c_str(), nboot[0], sd_threshold[0], probabilityM.c_str(),
         EMAlg.c_str(), stopping_threshold[0], log_stopping_threshold[0], verbose[0], maximum_iterations[0],
-        maximum_seconds[0], inputParams, &bestLogLL, &bestQ, &bestBootstrap, &bestTime, &finishReason, &totalIter);
+        maximum_seconds[0], &inputParams, &bestLogLL, &bestQ, &bestBootstrap, &bestTime, &finishReason, &totalIter);
+
+    if (inputParams.simulationMethod != nullptr)
+    {
+        free((void *)inputParams.simulationMethod);
+    }
 
     if (numCuts == 0) // Case where there's not any match
     {
@@ -328,6 +349,7 @@ Rcpp::List groupAggGreedy(Rcpp::String sd_statistic, Rcpp::NumericVector sd_thre
     free(bestQ);
     freeMatrix(&greedyP);
     freeMatrix(bestBootstrap);
+    Free(bestBootstrap);
     freeMatrix(&XR);
     freeMatrix(&WR);
 
